#!/bin/bash

LTE_RST_CHIP=
LTE_RST_PIN=
LTE_LED=
LTE_AT_PORT=
# DEV="cdc-wdm0"
IFNAME="wwan0"
CONNEECTION_NAME="wwan-lte-001"
I2C_BUS=10
I2C_ADD=20
PING_IP_1="223.5.5.5"
PING_IP_2="8.8.8.8"
is_ipc=
debug="false"
nm="true"
lte_metric=700

BSP_HOME_PATH=$(dirname $(realpath $0))
PY_SCRIPT="${BSP_HOME_PATH}/run-at-cmd.py"

source ${BSP_HOME_PATH}/custom-network-checking.sh

BASE_CONFIG_INI=${BSP_HOME_PATH}/ed-config.ini
function read_ini_file(){
    if [ ! -f $1 ];then
        return 1
    fi
    v=$(awk -F '=' '/\['$2'\]/{a=1}a==1&&$1~/'$3'/{print $2;exit}' $1)
    echo $v
    if [ ${#v} -eq 0 ];then
        return 1
    fi
    return 0
}

# 是否是NetworkManager 管理
function is_networkmanager(){
    if [ "x$nm" = "xtrue" ];then
        return 0
    fi
    return 1
}

function echo_error(){
    local t_msg=$*
    local t_time=$(date +%Y/%m/%d-%H:%M:%S)
    echo -e "[${t_time}]\t | [ERROR] | ${t_msg}"
}

function echo_info(){
    local t_msg=$*
    local t_time=$(date +%Y/%m/%d-%H:%M:%S)
    echo -e "[${t_time}]\t | [info] | ${t_msg}"
}

function die(){
    echo_error "$1"
    exit 1  
}

function do_lte_init_rst_pin() {
    if [ "x$is_ipc" = "xfalse" ];then
        LTE_RST_CHIP=0
        LTE_RST_PIN=10
    else
        LTE_RST_CHIP=$(gpiodetect | grep "${I2C_BUS}-00${I2C_ADD}" | sed "s/gpiochip//" | awk '{print $1}')
        [ $LTE_RST_CHIP ] || die "Can't read chip number of reset pin"
        LTE_RST_PIN=$(gpioinfo ${LTE_RST_CHIP} | grep "4G_RST" | cut -d ":" -f 1 | awk '{print $2}')
        [ $LTE_RST_PIN ] || die "Not found 4G reset pin"
        LTE_LED=$(gpioinfo ${LTE_RST_CHIP} | grep "4G_LED" | cut -d ":" -f 1 | awk '{print $2}')
        [ $LTE_LED ] || echo_info "Not found 4G LED pin"
        echo_info "CHIP=$LTE_RST_CHIP, RES_PIN=$LTE_RST_PIN, LED=$LTE_LED"
    fi

    m_4g_connection=$(read_ini_file $BASE_CONFIG_INI "4G" "connection")
    if [ $? -eq 0 ];then
        if [ ${#m_4g_connection} -gt 0 ];then
            CONNEECTION_NAME=$m_4g_connection
        fi
    fi
}

function lte_led_on(){
    [ ${LTE_LED} ] || return 0
    gpioset ${LTE_RST_CHIP} ${LTE_LED}=0
}

function lte_led_off(){
    [ ${LTE_LED} ] || return 0
    gpioset ${LTE_RST_CHIP} ${LTE_LED}=1
}

function do_lte_rst() {

    local t=0
    echo_info "reset lte module"
    
    if [[ "x$is_ipc" = "xfalse" ]];then
        gpioset ${LTE_RST_CHIP} ${LTE_RST_PIN}=1
        sleep 1
        gpioset ${LTE_RST_CHIP} ${LTE_RST_PIN}=0
    else
        gpioset ${LTE_RST_CHIP} ${LTE_RST_PIN}=0
        sleep 1
        gpioset ${LTE_RST_CHIP} ${LTE_RST_PIN}=1
    fi

    # it takes a while to load the device after reset
    sleep 5

    until [ -e /sys/class/net/${IFNAME} ]
    do
        sleep 1
        let t++
        if [ $t -gt 20 ];then
            lsusb | grep -q Quectel
            if [ $? -ne 0 ];then
                echo_error "Not found USB device"
                t=0
            fi
        fi
    done
    echo_info "finished reset lte module"
}

function update_udhcpc_script() {
    local udhcpc_script="/etc/udhcpc/default.script"
    if [ -f $udhcpc_script ];then
        sed -i "0,/^metric=.*/s|^metric=.*|metric=${lte_metric}|" $udhcpc_script
    fi
}
 
function do_connect() {
    is_networkmanager
    if [ $? -eq 0 ]; then
        systemctl is-active ModemManager.service > /dev/null
        [ $? -eq 0 ] || start_modem_service

        echo_info "Down connection ..."
        nmcli c delete ${CONNEECTION_NAME} > /dev/null 2>&1

        # echo_info "Reset lte module ..."
        # do_lte_rst
        # while true
        # do
        #     sleep 1
        #     nmcli device show ${DEV} >> /dev/null 2>&1
        #     if [ $? -eq 0 ];then
        #         break
        #     fi
        # done

        echo_info "Try to connect ..."
        nmcli connection show | grep $CONNEECTION_NAME  > /dev/null
        if [ $? -ne 0 ];then
            # create connection
            m_apn=$(read_ini_file $BASE_CONFIG_INI "4G" "apn")
            t_apn='nmcli connection add type gsm autoconnect yes con-name '$CONNEECTION_NAME
            t_metric="ipv4.route-metric ${lte_metric}"
            echo $m_apn | grep ',' > /dev/null
            if [ $? -eq 0 ];then
                m_array=(${m_apn//,/ })
                if [ ${#m_array[@]} -gt 0 ];then
                    t_apn=$t_apn' gsm.apn '$m_array[0]
                fi
                if [ ${#m_array[@]} -gt 1 ];then
                    t_apn=$t_apn' gsm.username "'$m_array[1]'"'
                fi
                if [ ${#m_array[@]} -gt 2 ];then
                    t_apn=$t_apn' gsm.password "'$m_array[2]'"'
                fi
            else
                if [ ${#m_apn} -gt 0 ];then
                    t_apn=$t_apn' gsm.apn '$m_apn
                fi
            fi
            echo_info "${CONNEECTION_NAME} is created"
            $t_apn $t_metric > /dev/null
        else
            nmcli c up ${CONNEECTION_NAME} > /dev/null 2>&1
        fi

        # nmcli device connect ${DEV}
        # systemctl restart NetworkManager.service
    else
        killall -q quectel-CM
        # echo_info "Reset lte module ..."
        # do_lte_rst
        echo_info "Try to connect ..."
        if [ ! -d "/var/log/ed-ec20-qmi/" ];then
            mkdir -p /var/log/ed-ec20-qmi/
        fi
        update_udhcpc_script
        m_apn=$(read_ini_file $BASE_CONFIG_INI "4G" "apn")
        if [ "x$m_apn" = "x" ];then
            q_apn=
        else
            q_apn="-s ${m_apn}"
        fi
        
        if [ "x$debug" = "xfalse" ];then
            /usr/share/ed-ec20-qmi/quectel-CM -4 -f /var/log/ed-ec20-qmi/quectel-CM.log $q_apn > /dev/null &
        else
            /usr/share/ed-ec20-qmi/quectel-CM -4 -f /var/log/ed-ec20-qmi/quectel-CM.log $q_apn &
        fi
    fi
}

function do_reset_connection(){
    local t_index=0
    local wait_time=(30 60 300)
    local reconnect_time=3

    check_conditions

    while [[ $t_index -lt ${reconnect_time} ]]
    do
        run_at_cmd "at+cfun=0"
        sleep 1
        run_at_cmd "at+cfun=1"
        sleep 1

        check_conditions
        do_connect
        sleep 20
        check_network_running
        if [ $? -eq 0 ];then
            break
        fi
        
        if [ ${t_index} -lt ${#wait_time[@]} ];then
            sleep "${wait_time[${t_index}]}"
        else
            sleep "${wait_time[2]}"
        fi
        
        let t_index++
    done

    if [[ $t_index -eq ${reconnect_time} ]];then
        do_lte_rst
        sleep 10

        check_conditions
        do_connect
        sleep 20
        check_network_running
        if [ $? -ne 0 ];then
            is_networkmanager
            if [ $? -eq 0 ];then
                set_apn_via_at
                do_connect
                sleep 10
            fi
        fi
    fi
}

function check_device_type(){
    dpkg -l | grep -E "ed-cm4(sen|ind)" > /dev/null
    if [ $? -eq 0 ];then
        is_ipc="false"
    else
        cat /sys/kernel/debug/gpio | grep "4G_RST" > /dev/null
        [ $? -eq 0 ] || sleep 5
        cat /sys/kernel/debug/gpio | grep "4G_RST" > /dev/null
        if [ $? -ne 0 ];then
            dpkg -l | grep -E "ed-(ipc|hmi).[12]|ed-plc" > /dev/null
            if [ $? -eq 0 ];then
                is_ipc="true"
                die "Reset pin cannot be found correctly"
            fi
            
            if [ "x$is_ipc" = "x" ];then
                die "Please install the corresponding BSP or Firmware of the product first"
            fi
        else
            is_ipc="true"
        fi
    fi

    if [ "x$is_ipc" = "xfalse" ];then
        echo_info "This device is CM4 Sensing or CM4 Industrial"
    else
        echo_info "This device is IPC, HMI or PLC"

        while true;do
            gpiodetect | grep "${I2C_BUS}-00${I2C_ADD}" > /dev/null
            if [ $? -eq 0 ];then
                break
            fi
            sleep 1
        done
    fi
}

function check_network_running(){
    declare -A m_servers
    m_whitelist=$(read_ini_file $BASE_CONFIG_INI "4G" "whitelist")
    [ "x$debug" = "xfalse" ] || echo_info "white list $m_whitelist"
    if [ $? -eq 0 ];then
        m_array=(${m_whitelist//;/ })
        for v in ${m_array[@]};
        do
            if [ ${#v} -ne 0 ];then
                m_servers[${#m_servers[@]}]=$v
            fi
        done
    else
        m_servers[0]=$PING_IP_1
        m_servers[1]=$PING_IP_2
    fi
    
    for i in {1..4};
    do
        for t_ip in ${m_servers[@]}
        do
            # ping -I ${IFNAME} -c 1 -s 0 ${t_ip} > /dev/null 2>&1
            check_network ${IFNAME} $t_ip
            if [ $? -eq 0 ]; then
                [ "x$is_ipc" = "xfalse" ]  || lte_led_on
                return 0;
            fi
        done
        sleep 1
    done
    [ "x$is_ipc" = "xfalse" ] || lte_led_off
    return 1;
}

function stop_modem_service(){
    systemctl stop ModemManager
    nmcli c down ${CONNEECTION_NAME} > /dev/null 2>&1
}

function start_modem_service(){
    systemctl start ModemManager
}

function reset_lte_module(){
    do_lte_rst
}

function check_conditions(){
    local fail_time=0

    echo_info "check_conditions"
    systemctl is-active ModemManager.service > /dev/null
    [ $? -ne 0 ] || stop_modem_service

    while true;do
        ls /dev/serial/by-id/usb-*-if03-port0 > /dev/null 2>&1
        if [ $? -eq 0 ];then
            LTE_AT_PORT=$(ls /dev/serial/by-id/usb-*-if03-port0)
            break
        fi
        if [ $fail_time -gt 60 ];then
            reset_lte_module

            fail_time=0
        fi
        sleep 2
        let fail_time++
    done

    while true;do
        check_SIM_status
        [ $? -ne 0 ] || break
        do_lte_rst
        sleep 150
    done

    while true;do
        check_signal
        [ $? -ne 0 ] || break
        sleep 30
    done
}

function check_SIM_status(){
    run_at_cmd "at+cpin?"
}

function check_signal(){
    run_at_cmd "at+csq"
}

function check_registration(){
    run_at_cmd "AT+CEREG?"
}

function set_apn_via_at(){
    local m_apn=$(read_ini_file $BASE_CONFIG_INI "4G" "apn")
    run_at_cmd "AT+CGDCONT=1,\"IP\",\"${m_apn}\""
}

function run_at_cmd(){
    local cmd=$*
    local verbose=
    if [ "x${debug}" = "xtrue" ];then
        verbose=" -v"
    fi
    python3 ${PY_SCRIPT} -d ${LTE_AT_PORT} -c ${cmd} ${verbose}
}

function do_start(){
    local m_auto=$(read_ini_file $BASE_CONFIG_INI "4G" "auto")
    local m_intervaltime=$(read_ini_file $BASE_CONFIG_INI "4G" "intervaltime")
    local m_debug=$(read_ini_file $BASE_CONFIG_INI "DEBUG" "debug")
    local m_nmcli=$(read_ini_file $BASE_CONFIG_INI "4G" "NetworkManager")
    local m_lte_metric=$(read_ini_file $BASE_CONFIG_INI "4G" "4g_metric")

    [ ! $m_debug ] || debug=$m_debug
    [ ! $m_nmcli ] || nm=$m_nmcli
    [ ! $m_lte_metric ] || lte_metric=$m_lte_metric
    
    echo_info "Auto start: $m_auto"
    echo_info "Interval time: $m_intervaltime"
    
    if [ "x${debug}" = "xtrue" ];then
        echo_info "Mode: DEBUG"
    else
        echo_info "Mode: Normal"
    fi

    is_networkmanager
    if [ $? -eq 0 ];then
        echo_info "Tools: NetworkManager"
    else
        echo_info "Tools: Quectel-CM"
    fi
    

    if [ $? -eq 0 ]; then
        if [ "$m_auto" = "1" ]; then
            check_device_type
            do_lte_init_rst_pin
            [ "x$is_ipc" = "xfalse" ] || lte_led_off
            check_conditions
            do_connect
            sleep 20
            index=0;
            while true; do
                check_network_running
                if [ $? -ne 0 ];then
                    if [ $index -lt 10 ];then
                        index=`expr $index + 1`
                    else
                        check_registration
                        check_conditions
                        do_reset_connection
                        index=0
                    fi
                else
                    index=0
                fi
                sleep ${m_intervaltime}
            done
        fi
    fi
}

if [ $(id -u) -ne 0 ]; then
  die "$0 must be run as root user"
fi
do_start