#!/bin/bash

BSP_HOME_PATH=$(dirname $(realpath $0))
nm="true"

BASE_CONFIG_INI=${BSP_HOME_PATH}/ed-config.ini
function read_ini_file(){
    if [ ! -f $1 ];then
        return 1
    fi
    v=$(awk -F '=' '/\['$2'\]/{a=1}a==1&&$1~/'$3'/{print $2;exit}' $1)
    echo $v
    if [ ${#src} -eq 0 ];then
        return 1
    fi
    return 0
}

function echo_error(){
    local t_msg=$*
    local t_time=$(date +%Y/%m/%d-%H:%M:%S)
    echo -e "[${t_time}]\t | [ERROR] | ${t_msg}"
}

function echo_info(){
    local t_msg=$*
    local t_time=$(date +%Y/%m/%d-%H:%M:%S)
    echo -e "[${t_time}]\t | [info] | ${t_msg}"
}

function die(){
    echo_error "$1"
    exit 1  
}

function is_networkmanager(){
    systemctl is-active NetworkManager > /dev/null
}

function check_nm_connect(){
    local ifname=$1
    nmcli connection show --active | \
        grep -E "ethernet|wifi|TYPE" | \
        grep -q "${ifname}"
}

function do_nm_modify(){
    local ifname=$1
    local ipv4_route_metric=$2

    if [ "x${ifname}" = "xwwan0" ];then
        systemctl restart ed-lte-tools
        return 0
    fi

    check_nm_connect ${ifname}
    if [ $? -ne 0 ];then
        echo_error "Not found connection ${ifname}"
        return 1
    fi

    uuid=$(nmcli c | grep $ifname | sed -n 's/.*\s\([0-9a-z]*-[0-9a-f]*-[0-9a-f]*-[0-9a-f]*-[0-9a-f]*\)\s.*/\1/p')
    if [ "x${uuid}" == "x" ];then
        echo_error "Can't find a connection that uses ${ifname}"
        return 1
    fi
    nmcli connection modify ${uuid} ipv4.route-metric ${ipv4_route_metric}
}

function update_nm_config(){
    systemctl restart NetworkManager
}

function read_net_metric(){
    t_net=$(read_ini_file $BASE_CONFIG_INI "Metric" "net${index}")
    if [ "x${t_net}" != "x" ];then
        t_net_array=(${t_net//,/ })
        site[${t_net_array[0]}]=${t_net_array[1]}
    fi
    unset t_net t_net_array
}

function do_start(){
    local m_nmcli=$(read_ini_file $BASE_CONFIG_INI "4G" "NetworkManager")
    declare -A site

    [ ! $m_nmcli ] || nm=$m_nmcli

    is_networkmanager
    if [ $? -ne 0 ];then
        echo_error "Only support NetworkManager"
        exit 1
    fi

    local index=0
    while true; do
        grep -E "net${index}" ${BASE_CONFIG_INI}
        grep -q -E "net${index}" ${BASE_CONFIG_INI}
        [ $? -eq 0 ] || break
        read_net_metric
        index=`expr $index + 1`
    done

    if [ $index -eq 0 ];then
        echo_info "No network metric config"
        exit 0
    fi

    for ifname in ${!site[*]}; do
        if [ "x${site[${ifname}]}" == "x" ];then
            echo_error "${ifname} no metric set"
            continue
        fi
        echo_info ${ifname} ${site[${ifname}]}
        do_nm_modify ${ifname} ${site[${ifname}]}
    done
    update_nm_config

    unset site
}

if [ $(id -u) -ne 0 ]; then
  die "$0 must be run as root user"
fi
do_start